require "rd-test-util"

require 'rwiki/rd/ext/block-verbatim'
require 'rwiki/rd/ext/enscript'

class TestRDExtBlockVerbatim < Test::Unit::TestCase
  include RDTestUtil

  def test_ext_block_verb_erb_escape
    source = <<-EOS
  <%= 1 + 1 %>
EOS

    expected_source = <<-EOS.rstrip
<pre>
&lt;%= 1 + 1 %&gt;
</pre>
EOS

    expected = HTree.parse(expected_source)
    actual = HTree.parse(parse_rd(source))
    assert_equal(expected, actual)
  end

  def test_ext_block_verb_enscript
    return unless RD::Ext::BlockVerbatim.enscript_available?

    enscript_version = StringIO.new(`enscript --version`).gets.chomp
    
    source = <<-'EOS'
  # enscript c
  #include <stdio.h>
  int
  main(int argc, char **argv, char **env)
  {
    printf("Hello World!\n");
    return 0;
  }
EOS

    expected_source = <<-EOS.rstrip
<pre class="enscript">
#<span style="font-weight: bold"><span style="color: #5f9ea0">include</span></span> <span style="font-weight: bold"><span style="color: #bc8f8f">&lt;stdio.h&gt;</span></span>
<span style="font-weight: bold"><span style="color: #228b22">int</span></span>
<span style="font-weight: bold"><span style="color: #0000ff">main</span></span>(<span style="font-weight: bold"><span style="color: #228b22">int</span></span> argc, <span style="font-weight: bold"><span style="color: #228b22">char</span></span> **argv, <span style="font-weight: bold"><span style="color: #228b22">char</span></span> **env)
{
  printf(<span style="font-weight: bold"><span style="color: #bc8f8f">&quot;Hello World!\\n&quot;</span></span>);
  <span style="font-weight: bold"><span style="color: #a020f0">return</span></span> 0;
}
</pre>
<!-- <address>Generated by <a href="http://www.iki.fi/~mtr/genscript/">#{enscript_version}</a>.</address> -->
EOS
    
    expected = HTree.parse(expected_source)
    actual = HTree.parse(parse_rd(source))
    assert_equal(expected, actual)
  end
end
